/*
 * This file is automatically generated.  DON'T EDIT THIS FILE MANUALLY.
 */

#include "sombok_constants.h"
#include "sombok.h"
#define UNICODE_VERSION "6.1.0"
const char *linebreak_unicode_version = UNICODE_VERSION;

const char *linebreak_propvals_EA[] = {
    "Z", "Na", "N", "A", "W", "H", "F", NULL
};
const char *linebreak_propvals_GB[] = {
    "CR", "LF", "Control", "Extend", "Prepend", "SpacingMark", "L", "V", 
    "T", "LV", "LVT", "Other", "Virama", "OtherLetter", NULL
};
const char *linebreak_propvals_LB[] = {
    "BK", "CR", "LF", "NL", "SP", "OP", "CL", "QU", "GL", "NS", "EX", "SY", 
    "IS", "PR", "PO", "NU", "AL", "ID", "IN", "HY", "BA", "BB", "B2", "CB", 
    "ZW", "CM", "WJ", "H2", "H3", "JL", "JV", "JT", "CP", "SG", "AI", "SA", 
    "HL", "CJ", "XX", NULL
};
const char *linebreak_propvals_SC[] = {
    "Common", "Inherited", "Unknown", "Han", "Hangul", "Thai", "Lao", 
    "Myanmar", "Khmer", "Tai_Le", "New_Tai_Lue", "Tai_Tham", "Tai_Viet", 
    NULL
};

#define M (LINEBREAK_ACTION_MANDATORY)
#define D (LINEBREAK_ACTION_DIRECT)
#define I (LINEBREAK_ACTION_INDIRECT)
#define P (LINEBREAK_ACTION_PROHIBITED)
#define r(cc) static propval_t rule_##cc[]
/* Note: Entries related to BK, CR, CM, LF, NL aren't used by break(). */
    /* BkCrLfNlSpOpClQuGlNsExSyIsPrPoNuAlIdInHyBaBbB2CbZwCmWjH2H3JlJvJtCp */
r(BK)={M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M};
r(CR)={M,M,P,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M};
r(LF)={M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M};
r(NL)={M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M};
r(SP)={P,P,P,P,P,D,P,D,D,D,P,P,P,D,D,D,D,D,D,D,D,D,D,D,P,D,P,D,D,D,D,D,P};
r(OP)={P,P,P,P,P,P,P,P,P,P,P,P,P,P,P,P,P,P,P,P,P,P,P,P,P,P,P,P,P,P,P,P,P};
r(CL)={P,P,P,P,P,D,P,I,I,P,P,P,P,D,D,D,D,D,D,I,I,D,D,D,P,D,P,D,D,D,D,D,P};
r(QU)={P,P,P,P,P,P,P,I,I,I,P,P,P,I,I,I,I,I,I,I,I,I,I,I,P,I,P,I,I,I,I,I,P};
r(GL)={P,P,P,P,P,I,P,I,I,I,P,P,P,I,I,I,I,I,I,I,I,I,I,I,P,I,P,I,I,I,I,I,P};
r(NS)={P,P,P,P,P,D,P,I,I,I,P,P,P,D,D,D,D,D,D,I,I,D,D,D,P,D,P,D,D,D,D,D,P};
r(EX)={P,P,P,P,P,D,P,I,I,I,P,P,P,D,D,D,D,D,D,I,I,D,D,D,P,D,P,D,D,D,D,D,P};
r(SY)={P,P,P,P,P,D,P,I,I,I,P,P,P,D,D,D,D,D,D,I,I,D,D,D,P,D,P,D,D,D,D,D,P};
r(IS)={P,P,P,P,P,D,P,I,I,I,P,P,P,D,D,D,I,D,D,I,I,D,D,D,P,D,P,D,D,D,D,D,P};
r(PR)={P,P,P,P,P,D,P,I,I,I,P,P,P,D,D,D,I,I,D,I,I,D,D,D,P,D,P,I,I,I,I,I,P};
r(PO)={P,P,P,P,P,D,P,I,I,I,P,P,P,D,D,D,I,D,D,I,I,D,D,D,P,D,P,D,D,D,D,D,P};
r(NU)={P,P,P,P,P,I,P,I,I,I,P,P,P,D,D,D,I,D,I,I,I,D,D,D,P,D,P,D,D,D,D,D,P};
r(AL)={P,P,P,P,P,I,P,I,I,I,P,P,P,D,D,I,I,D,I,I,I,D,D,D,P,D,P,D,D,D,D,D,P};
r(ID)={P,P,P,P,P,D,P,I,I,I,P,P,P,D,I,D,D,D,I,I,I,D,D,D,P,D,P,D,D,D,D,D,P};
r(IN)={P,P,P,P,P,D,P,I,I,I,P,P,P,D,D,D,D,D,I,I,I,D,D,D,P,D,P,D,D,D,D,D,P};
r(HY)={P,P,P,P,P,D,P,I,D,I,P,P,P,D,D,D,D,D,D,I,I,D,D,D,P,D,P,D,D,D,D,D,P};
r(BA)={P,P,P,P,P,D,P,I,D,I,P,P,P,D,D,D,D,D,D,I,I,D,D,D,P,D,P,D,D,D,D,D,P};
r(BB)={P,P,P,P,P,I,P,I,I,I,P,P,P,I,I,I,I,I,I,I,I,I,I,D,P,I,P,I,I,I,I,I,P};
r(B2)={P,P,P,P,P,D,P,I,I,I,P,P,P,D,D,D,D,D,D,I,I,D,P,D,P,D,P,D,D,D,D,D,P};
r(CB)={P,P,P,P,P,D,P,I,I,D,P,P,P,D,D,D,D,D,D,D,D,D,D,D,P,D,P,D,D,D,D,D,P};
r(ZW)={P,P,P,P,P,D,D,D,D,D,D,D,D,D,D,D,D,D,D,D,D,D,D,D,P,D,D,D,D,D,D,D,D};
r(CM)={P,P,P,P,P,D,P,I,I,I,P,P,P,D,D,D,D,D,D,I,I,D,D,D,P,D,P,D,D,D,D,D,P};
r(WJ)={P,P,P,P,P,I,P,I,I,I,P,P,P,I,I,I,I,I,I,I,I,I,I,I,P,I,P,I,I,I,I,I,P};
r(H2)={P,P,P,P,P,D,P,I,I,I,P,P,P,D,I,D,D,D,I,I,I,D,D,D,P,D,P,D,D,D,I,I,P};
r(H3)={P,P,P,P,P,D,P,I,I,I,P,P,P,D,I,D,D,D,I,I,I,D,D,D,P,D,P,D,D,D,D,I,P};
r(JL)={P,P,P,P,P,D,P,I,I,I,P,P,P,D,I,D,D,D,I,I,I,D,D,D,P,D,P,I,I,I,I,D,P};
r(JV)={P,P,P,P,P,D,P,I,I,I,P,P,P,D,I,D,D,D,I,I,I,D,D,D,P,D,P,D,D,D,I,I,P};
r(JT)={P,P,P,P,P,D,P,I,I,I,P,P,P,D,I,D,D,D,I,I,I,D,D,D,P,D,P,D,D,D,D,I,P};
r(CP)={P,P,P,P,P,D,P,I,I,P,P,P,P,D,D,I,I,D,D,I,I,D,D,D,P,D,P,D,D,D,D,D,P};
#undef r
propval_t *linebreak_rules[] = {
    rule_BK, rule_CR, rule_LF, rule_NL, rule_SP, rule_OP, rule_CL, rule_QU, 
    rule_GL, rule_NS, rule_EX, rule_SY, rule_IS, rule_PR, rule_PO, rule_NU, 
    rule_AL, rule_ID, rule_IN, rule_HY, rule_BA, rule_BB, rule_B2, rule_CB, 
    rule_ZW, rule_CM, rule_WJ, rule_H2, rule_H3, rule_JL, rule_JV, rule_JT, 
    rule_CP
};

size_t linebreak_rulessiz = 33;

unsigned short linebreak_prop_index[] = {
    0, 32, 64, 94, 125, 157, 189, 219, 248, 276, 307, 333, 345, 345, 363,
    345, 345, 345, 392, 343, 345, 345, 420, 345, 452, 452, 469, 498, 529,
    559, 589, 345, 620, 636, 652, 345, 681, 345, 345, 345, 345, 345, 345,
    345, 704, 721, 753, 774, 806, 345, 838, 870, 345, 345, 887, 918, 944,
    962, 983, 345, 345, 1009, 1041, 1062, 1088, 1116, 1130, 345, 345, 345,
    345, 1158, 1190, 1196, 1227, 1257, 1289, 1312, 1343, 1374, 1405, 1428,
    1459, 1490, 1521, 1544, 1575, 1607, 1289, 1639, 1671, 1702, 1732, 1764,
    1796, 1822, 1853, 1876, 1907, 1939, 1969, 1992, 2024, 2056, 1969, 2088,
    2119, 2151, 2183, 2209, 2241, 2273, 2304, 2319, 2351, 345, 2382, 2414,
    2446, 345, 2478, 2510, 2542, 2561, 2593, 2618, 2650, 345, 2682, 2703,
    2735, 2766, 2796, 345, 345, 345, 2828, 2828, 2828, 2860, 2860, 2889,
    2897, 2903, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 2935, 2967,
    345, 345, 345, 345, 2968, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 3000, 345, 345, 3029,
    3061, 3093, 3116, 3116, 3148, 3160, 3190, 1041, 3220, 345, 345, 345, 345,
    3246, 345, 345, 345, 3278, 3306, 3324, 3356, 3376, 3403, 345, 3435, 3467,
    3478, 3510, 3542, 3574, 345, 345, 3606, 3618, 3648, 3679, 3711, 3742,
    3768, 3794, 345, 3822, 3542, 3854, 345, 345, 3886, 3917, 345, 345, 345,
    345, 345, 345, 452, 987, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 3938, 3970, 4002, 4033, 4065, 4097, 4129,
    962, 4161, 4190, 4221, 4234, 4266, 4292, 4318, 4344, 4369, 4401, 4432,
    4462, 4494, 4524, 4551, 345, 345, 4564, 4587, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 4619, 4619, 4619, 4619, 4641, 4619, 4619, 4673, 4689,
    4721, 4753, 4781, 4811, 4838, 345, 4866, 4898, 4914, 4914, 4940, 4970,
    345, 5002, 5032, 5063, 5095, 345, 5122, 5148, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 5177, 345, 5202, 5230, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 5260, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 5286, 345, 5318, 3768, 5342, 345, 345, 345,
    452, 5374, 5406, 345, 345, 5438, 5465, 5465, 5477, 5465, 5465, 5465,
    5465, 5465, 5465, 5475, 5497, 5529, 5561, 5593, 5622, 5651, 5683, 5622,
    5712, 5744, 5762, 5465, 5465, 5449, 5470, 5465, 5790, 5466, 5465, 5822,
    5465, 5465, 5465, 5465, 5466, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 345, 345, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5838, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    5465, 5860, 5465, 5885, 3854, 345, 345, 345, 345, 345, 345, 345, 345,
    5904, 1041, 345, 5921, 5951, 345, 345, 5966, 345, 345, 345, 345, 345,
    345, 345, 345, 5998, 6027, 345, 6052, 6084, 6096, 6124, 6156, 1041, 6184,
    6209, 6241, 1190, 6273, 6305, 345, 345, 6331, 6360, 6392, 6424, 6440,
    6472, 6504, 345, 345, 345, 345, 345, 345, 3768, 6536, 6568, 6572, 6576,
    6580, 6584, 6588, 6592, 6568, 6572, 6576, 6580, 6584, 6588, 6592, 6568,
    6572, 6576, 6580, 6584, 6588, 6592, 6568, 6572, 6576, 6580, 6584, 6588,
    6592, 6568, 6572, 6576, 6580, 6584, 6588, 6592, 6568, 6572, 6576, 6580,
    6584, 6588, 6592, 6568, 6572, 6576, 6580, 6584, 6588, 6592, 6568, 6572,
    6576, 6580, 6584, 6588, 6592, 6568, 6572, 6576, 6580, 6584, 6588, 6592,
    6568, 6572, 6576, 6580, 6584, 6588, 6592, 6568, 6572, 6576, 6580, 6584,
    6588, 6592, 6568, 6572, 6576, 6580, 6584, 6588, 6592, 6568, 6572, 6576,
    6580, 6584, 6588, 6592, 6568, 6572, 6576, 6580, 6584, 6588, 6592, 6568,
    6572, 6576, 6580, 6584, 6588, 6592, 6568, 6572, 6576, 6580, 6584, 6588,
    6592, 6568, 6572, 6576, 6580, 6584, 6588, 6592, 6568, 6572, 6576, 6580,
    6584, 6588, 6592, 6568, 6572, 6576, 6580, 6584, 6588, 6592, 6568, 6572,
    6576, 6580, 6584, 6588, 6592, 6568, 6572, 6576, 6580, 6584, 6588, 6592,
    6568, 6572, 6576, 6580, 6584, 6588, 6592, 6568, 6572, 6576, 6580, 6584,
    6588, 6592, 6568, 6572, 6576, 6580, 6584, 6588, 6592, 6568, 6572, 6576,
    6580, 6584, 6588, 6592, 6568, 6572, 6576, 6580, 6584, 6588, 6592, 6568,
    6572, 6576, 6580, 6584, 6588, 6592, 6568, 6572, 6576, 6580, 6584, 6588,
    6592, 6568, 6572, 6576, 6580, 6584, 6588, 6592, 6568, 6572, 6576, 6580,
    6584, 6588, 6592, 6568, 6572, 6576, 6580, 6584, 6588, 6592, 6568, 6572,
    6576, 6580, 6584, 6588, 6592, 6568, 6572, 6576, 6580, 6584, 6588, 6592,
    6568, 6572, 6576, 6580, 6584, 6588, 6592, 6568, 6572, 6576, 6580, 6584,
    6588, 6592, 6568, 6572, 6576, 6580, 6584, 6588, 6592, 6568, 6572, 6576,
    6580, 6584, 6588, 6592, 6568, 6572, 6576, 6580, 6584, 6588, 6592, 6568,
    6572, 6576, 6580, 6584, 6588, 6592, 6568, 6572, 6576, 6580, 6584, 6588,
    6592, 6568, 6572, 6576, 6580, 6584, 6588, 6592, 6568, 6572, 6576, 6580,
    6584, 6588, 6592, 6568, 6572, 6576, 6580, 6584, 6588, 6592, 6568, 6572,
    6576, 6580, 6584, 6588, 6592, 6568, 6572, 6576, 6580, 6584, 6588, 6592,
    6568, 6572, 6576, 6580, 6584, 6588, 6592, 6568, 6572, 6576, 6580, 6584,
    6588, 6592, 6568, 6572, 6576, 6580, 6584, 6588, 6592, 6568, 6572, 6576,
    6580, 6584, 6588, 6592, 6568, 6572, 6576, 6580, 6584, 6588, 6620, 6645,
    6656, 6688, 6688, 6688, 6688, 6688, 6688, 6688, 6688, 6688, 6688, 6688,
    6688, 6688, 6688, 6688, 6688, 6688, 6688, 6688, 6688, 6688, 6688, 6688,
    6688, 6688, 6688, 6688, 6688, 6688, 6688, 6688, 6688, 6688, 6688, 6688,
    6688, 6688, 6688, 6688, 6688, 6688, 6688, 6688, 6688, 6688, 6688, 6688,
    6688, 6688, 6688, 6688, 6688, 6688, 6688, 6688, 6688, 6688, 6688, 6688,
    6688, 6688, 6688, 6688, 6688, 636, 636, 636, 636, 636, 636, 636, 636,
    636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636,
    636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636,
    636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636,
    636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636,
    636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636,
    636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636,
    636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636,
    636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636,
    636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636,
    636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636,
    636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636,
    636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636,
    636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636,
    636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 5465, 5465, 5465, 5465,
    5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465, 5465,
    6720, 6751, 6783, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 6799, 345, 345, 345, 345, 345, 6831, 6863, 6895, 6923,
    6954, 345, 345, 345, 6966, 6998, 7030, 7060, 7092, 7109, 7141, 7172,
    7204, 345, 345, 345, 345, 345, 345, 345, 345, 3851, 345, 345, 345, 345,
    345, 345, 7234, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 2969, 345, 2984, 345, 345, 345, 345, 345, 345, 1041, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 2977, 345, 345,
    345, 345, 345, 2969, 345, 345, 345, 345, 345, 345, 345, 7266, 7291, 7323,
    345, 345, 345, 345, 345, 345, 7347, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 7379, 7387, 7413, 1035, 3711, 7445, 3852, 7477, 7509,
    7534, 3850, 345, 3711, 7566, 7598, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 3768, 7630, 1041, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 7654, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    7674, 345, 7704, 345, 345, 345, 345, 345, 345, 7714, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 7742, 7759,
    7790, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 7822, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 7849,
    7878, 7900, 345, 345, 345, 345, 7930, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 7948, 7962, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    7994, 8012, 4619, 8034, 8050, 345, 345, 345, 8082, 5470, 8105, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345
};

propval_t linebreak_prop_array[] = {
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_BA, EA_Z, GB_Control, SC_Unknown,
    LB_LF, EA_Z, GB_LF, SC_Unknown, LB_BK, EA_Z, GB_Control, SC_Unknown,
    LB_BK, EA_Z, GB_Control, SC_Unknown, LB_CR, EA_Z, GB_CR, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_SP, EA_Na, GB_Other, SC_Unknown, LB_EX, EA_Na, GB_Other, SC_Unknown,
    LB_QU, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_PR, EA_Na, GB_Other, SC_Unknown, LB_PO, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_QU, EA_Na, GB_Other, SC_Unknown,
    LB_OP, EA_Na, GB_Other, SC_Unknown, LB_CP, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_PR, EA_Na, GB_Other, SC_Unknown,
    LB_IS, EA_Na, GB_Other, SC_Unknown, LB_HY, EA_Na, GB_Other, SC_Unknown,
    LB_IS, EA_Na, GB_Other, SC_Unknown, LB_SY, EA_Na, GB_Other, SC_Unknown,
    LB_NU, EA_Na, GB_Other, SC_Unknown, LB_NU, EA_Na, GB_Other, SC_Unknown,
    LB_NU, EA_Na, GB_Other, SC_Unknown, LB_NU, EA_Na, GB_Other, SC_Unknown,
    LB_NU, EA_Na, GB_Other, SC_Unknown, LB_NU, EA_Na, GB_Other, SC_Unknown,
    LB_NU, EA_Na, GB_Other, SC_Unknown, LB_NU, EA_Na, GB_Other, SC_Unknown,
    LB_NU, EA_Na, GB_Other, SC_Unknown, LB_NU, EA_Na, GB_Other, SC_Unknown,
    LB_IS, EA_Na, GB_Other, SC_Unknown, LB_IS, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_EX, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_OP, EA_Na, GB_Other, SC_Unknown,
    LB_PR, EA_Na, GB_Other, SC_Unknown, LB_CP, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_OP, EA_Na, GB_Other, SC_Unknown,
    LB_BA, EA_Na, GB_Other, SC_Unknown, LB_CL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_NL, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_GL, EA_N, GB_Other, SC_Unknown,
    LB_OP, EA_A, GB_Other, SC_Unknown, LB_PO, EA_Na, GB_Other, SC_Unknown,
    LB_PR, EA_Na, GB_Other, SC_Unknown, LB_PR, EA_A, GB_Other, SC_Unknown,
    LB_PR, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_QU, EA_N, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_BA, EA_Z, GB_Control, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_PO, EA_A, GB_Other, SC_Unknown,
    LB_PR, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_BB, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_QU, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_OP, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_BB, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_BB, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_BB, EA_A, GB_Other, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_GL, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_GL, EA_Z, GB_Extend, SC_Unknown,
    LB_GL, EA_Z, GB_Extend, SC_Unknown, LB_GL, EA_Z, GB_Extend, SC_Unknown,
    LB_GL, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_IS, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_IS, EA_N, GB_Other, SC_Unknown,
    LB_BA, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_PR, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_EX, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_HL, EA_N, GB_Other, SC_Unknown,
    LB_HL, EA_N, GB_Other, SC_Unknown, LB_HL, EA_N, GB_Other, SC_Unknown,
    LB_HL, EA_N, GB_Other, SC_Unknown, LB_HL, EA_N, GB_Other, SC_Unknown,
    LB_HL, EA_N, GB_Other, SC_Unknown, LB_HL, EA_N, GB_Other, SC_Unknown,
    LB_HL, EA_N, GB_Other, SC_Unknown, LB_HL, EA_N, GB_Other, SC_Unknown,
    LB_HL, EA_N, GB_Other, SC_Unknown, LB_HL, EA_N, GB_Other, SC_Unknown,
    LB_HL, EA_N, GB_Other, SC_Unknown, LB_HL, EA_N, GB_Other, SC_Unknown,
    LB_HL, EA_N, GB_Other, SC_Unknown, LB_HL, EA_N, GB_Other, SC_Unknown,
    LB_HL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_HL, EA_N, GB_Other, SC_Unknown, LB_HL, EA_N, GB_Other, SC_Unknown,
    LB_HL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_Z, GB_Control, SC_Unknown, LB_AL, EA_Z, GB_Control, SC_Unknown,
    LB_AL, EA_Z, GB_Control, SC_Unknown, LB_AL, EA_Z, GB_Control, SC_Unknown,
    LB_AL, EA_Z, GB_Control, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_PO, EA_N, GB_Other, SC_Unknown,
    LB_PO, EA_N, GB_Other, SC_Unknown, LB_PO, EA_N, GB_Other, SC_Unknown,
    LB_IS, EA_N, GB_Other, SC_Unknown, LB_IS, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_EX, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_EX, EA_N, GB_Other, SC_Unknown, LB_EX, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_PO, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_EX, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_AL, EA_Z, GB_Control, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_Z, GB_Control, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_IS, EA_N, GB_Other, SC_Unknown, LB_EX, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_Z, GB_Virama, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL,
    EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL,
    EA_N, GB_OtherLetter, SC_Unknown, LB_CM, EA_N, GB_Extend, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_Z, GB_Virama, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_CM, EA_N, GB_Extend, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_PO, EA_N, GB_Other, SC_Unknown, LB_PO, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_PO, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_PR, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL,
    EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL,
    EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Virama, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N,
    GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N,
    GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N,
    GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N,
    GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N,
    GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL,
    EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL,
    EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_CM,
    EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM,
    EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_Z, GB_Virama, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_PR,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL,
    EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_CM, EA_N, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_Z, GB_Virama, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_N, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N,
    GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N,
    GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N,
    GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N,
    GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N,
    GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_CM, EA_N, GB_Extend, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_Z, GB_Virama, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_CM, EA_N, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_PR, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Virama, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL,
    EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL,
    EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL,
    EA_N, GB_OtherLetter, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_N, GB_Extend, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Virama, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_CM, EA_N, GB_Extend, SC_Unknown, LB_CM, EA_N,
    GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_CM, EA_N, GB_Extend, SC_Unknown, LB_CM,
    EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM,
    EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_Z, GB_Virama, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_CM, EA_N, GB_Extend, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_PO, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL,
    EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Virama,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_CM, EA_N, GB_Extend, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_N, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_SA, EA_N, GB_OtherLetter, SC_Thai, LB_SA, EA_N, GB_OtherLetter,
    SC_Thai, LB_SA, EA_N, GB_OtherLetter, SC_Thai, LB_SA, EA_N,
    GB_OtherLetter, SC_Thai, LB_SA, EA_N, GB_OtherLetter, SC_Thai, LB_SA,
    EA_N, GB_OtherLetter, SC_Thai, LB_SA, EA_N, GB_OtherLetter, SC_Thai,
    LB_SA, EA_N, GB_OtherLetter, SC_Thai, LB_SA, EA_N, GB_OtherLetter,
    SC_Thai, LB_SA, EA_N, GB_OtherLetter, SC_Thai, LB_SA, EA_N,
    GB_OtherLetter, SC_Thai, LB_SA, EA_N, GB_OtherLetter, SC_Thai, LB_SA,
    EA_N, GB_OtherLetter, SC_Thai, LB_SA, EA_N, GB_OtherLetter, SC_Thai,
    LB_SA, EA_N, GB_OtherLetter, SC_Thai, LB_SA, EA_N, GB_OtherLetter,
    SC_Thai, LB_SA, EA_N, GB_OtherLetter, SC_Thai, LB_SA, EA_N,
    GB_OtherLetter, SC_Thai, LB_SA, EA_N, GB_OtherLetter, SC_Thai, LB_SA,
    EA_N, GB_OtherLetter, SC_Thai, LB_SA, EA_N, GB_OtherLetter, SC_Thai,
    LB_SA, EA_N, GB_OtherLetter, SC_Thai, LB_SA, EA_N, GB_OtherLetter,
    SC_Thai, LB_SA, EA_N, GB_OtherLetter, SC_Thai, LB_SA, EA_N,
    GB_OtherLetter, SC_Thai, LB_SA, EA_N, GB_OtherLetter, SC_Thai, LB_SA,
    EA_N, GB_OtherLetter, SC_Thai, LB_SA, EA_N, GB_OtherLetter, SC_Thai,
    LB_SA, EA_N, GB_OtherLetter, SC_Thai, LB_SA, EA_N, GB_OtherLetter,
    SC_Thai, LB_SA, EA_N, GB_OtherLetter, SC_Thai, LB_SA, EA_Z, GB_Extend,
    SC_Thai, LB_SA, EA_N, GB_OtherLetter, SC_Thai, LB_SA, EA_N,
    GB_SpacingMark, SC_Thai, LB_SA, EA_Z, GB_Extend, SC_Thai, LB_SA, EA_Z,
    GB_Extend, SC_Thai, LB_SA, EA_Z, GB_Extend, SC_Thai, LB_SA, EA_Z,
    GB_Extend, SC_Thai, LB_SA, EA_Z, GB_Extend, SC_Thai, LB_SA, EA_Z,
    GB_Extend, SC_Thai, LB_SA, EA_Z, GB_Virama, SC_Thai, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_PR, EA_N,
    GB_Other, SC_Unknown, LB_SA, EA_N, GB_OtherLetter, SC_Thai, LB_SA, EA_N,
    GB_OtherLetter, SC_Thai, LB_SA, EA_N, GB_OtherLetter, SC_Thai, LB_SA,
    EA_N, GB_OtherLetter, SC_Thai, LB_SA, EA_N, GB_OtherLetter, SC_Thai,
    LB_SA, EA_N, GB_OtherLetter, SC_Thai, LB_SA, EA_N, GB_OtherLetter,
    SC_Thai, LB_SA, EA_Z, GB_Extend, SC_Thai, LB_SA, EA_Z, GB_Extend,
    SC_Thai, LB_SA, EA_Z, GB_Extend, SC_Thai, LB_SA, EA_Z, GB_Extend,
    SC_Thai, LB_SA, EA_Z, GB_Extend, SC_Thai, LB_SA, EA_Z, GB_Extend,
    SC_Thai, LB_SA, EA_Z, GB_Extend, SC_Thai, LB_SA, EA_Z, GB_Extend,
    SC_Thai, LB_AL, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other,
    SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_SA, EA_N, GB_Other,
    SC_Lao, LB_SA, EA_N, GB_Other, SC_Lao, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_SA, EA_N, GB_Other, SC_Lao, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_SA, EA_N, GB_Other, SC_Lao, LB_SA, EA_N,
    GB_Other, SC_Lao, LB_AL, EA_N, GB_Other, SC_Unknown, LB_SA, EA_N,
    GB_Other, SC_Lao, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_SA, EA_N, GB_Other, SC_Lao, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_SA, EA_N,
    GB_Other, SC_Lao, LB_SA, EA_N, GB_Other, SC_Lao, LB_SA, EA_N, GB_Other,
    SC_Lao, LB_SA, EA_N, GB_Other, SC_Lao, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_SA, EA_N, GB_Other, SC_Lao, LB_SA, EA_N, GB_Other, SC_Lao, LB_SA,
    EA_N, GB_Other, SC_Lao, LB_SA, EA_N, GB_Other, SC_Lao, LB_SA, EA_N,
    GB_Other, SC_Lao, LB_SA, EA_N, GB_Other, SC_Lao, LB_SA, EA_N, GB_Other,
    SC_Lao, LB_AL, EA_N, GB_Other, SC_Unknown, LB_SA, EA_N, GB_Other, SC_Lao,
    LB_SA, EA_N, GB_Other, SC_Lao, LB_SA, EA_N, GB_Other, SC_Lao, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_SA, EA_N, GB_Other, SC_Lao, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_SA, EA_N, GB_Other, SC_Lao, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_SA, EA_N,
    GB_Other, SC_Lao, LB_SA, EA_N, GB_Other, SC_Lao, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_SA, EA_N, GB_Other, SC_Lao, LB_SA, EA_N, GB_Other, SC_Lao,
    LB_SA, EA_N, GB_Other, SC_Lao, LB_SA, EA_N, GB_Other, SC_Lao, LB_SA,
    EA_Z, GB_Extend, SC_Lao, LB_SA, EA_N, GB_Other, SC_Lao, LB_SA, EA_N,
    GB_SpacingMark, SC_Lao, LB_SA, EA_Z, GB_Extend, SC_Lao, LB_SA, EA_Z,
    GB_Extend, SC_Lao, LB_SA, EA_Z, GB_Extend, SC_Lao, LB_SA, EA_Z,
    GB_Extend, SC_Lao, LB_SA, EA_Z, GB_Extend, SC_Lao, LB_SA, EA_Z,
    GB_Extend, SC_Lao, LB_AL, EA_N, GB_Other, SC_Unknown, LB_SA, EA_Z,
    GB_Extend, SC_Lao, LB_SA, EA_Z, GB_Extend, SC_Lao, LB_SA, EA_N, GB_Other,
    SC_Lao, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_SA, EA_N, GB_Other, SC_Lao, LB_SA, EA_N, GB_Other, SC_Lao,
    LB_SA, EA_N, GB_Other, SC_Lao, LB_SA, EA_N, GB_Other, SC_Lao, LB_SA,
    EA_N, GB_Other, SC_Lao, LB_AL, EA_N, GB_Other, SC_Unknown, LB_SA, EA_N,
    GB_Other, SC_Lao, LB_AL, EA_N, GB_Other, SC_Unknown, LB_SA, EA_Z,
    GB_Extend, SC_Lao, LB_SA, EA_Z, GB_Extend, SC_Lao, LB_SA, EA_Z,
    GB_Extend, SC_Lao, LB_SA, EA_Z, GB_Extend, SC_Lao, LB_SA, EA_Z,
    GB_Extend, SC_Lao, LB_SA, EA_Z, GB_Extend, SC_Lao, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_SA, EA_N, GB_Other,
    SC_Lao, LB_SA, EA_N, GB_Other, SC_Lao, LB_SA, EA_N, GB_Other, SC_Lao,
    LB_SA, EA_N, GB_Other, SC_Lao, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_BB, EA_N, GB_Other, SC_Unknown, LB_BB, EA_N, GB_Other, SC_Unknown,
    LB_BB, EA_N, GB_Other, SC_Unknown, LB_BB, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_BB, EA_N, GB_Other, SC_Unknown,
    LB_BB, EA_N, GB_Other, SC_Unknown, LB_GL, EA_N, GB_Other, SC_Unknown,
    LB_BB, EA_N, GB_Other, SC_Unknown, LB_BB, EA_N, GB_Other, SC_Unknown,
    LB_BA, EA_N, GB_Other, SC_Unknown, LB_GL, EA_N, GB_Other, SC_Unknown,
    LB_EX, EA_N, GB_Other, SC_Unknown, LB_EX, EA_N, GB_Other, SC_Unknown,
    LB_EX, EA_N, GB_Other, SC_Unknown, LB_EX, EA_N, GB_Other, SC_Unknown,
    LB_EX, EA_N, GB_Other, SC_Unknown, LB_GL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_EX, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_OP, EA_N, GB_Other, SC_Unknown,
    LB_CL, EA_N, GB_Other, SC_Unknown, LB_OP, EA_N, GB_Other, SC_Unknown,
    LB_CL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_BA, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Virama,
    SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N,
    GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_BB, EA_N,
    GB_Other, SC_Unknown, LB_BB, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N,
    GB_Other, SC_Unknown, LB_BB, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_GL, EA_N, GB_Other, SC_Unknown, LB_GL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_SA, EA_N,
    GB_OtherLetter, SC_Myanmar, LB_SA, EA_N, GB_OtherLetter, SC_Myanmar,
    LB_SA, EA_N, GB_OtherLetter, SC_Myanmar, LB_SA, EA_N, GB_OtherLetter,
    SC_Myanmar, LB_SA, EA_N, GB_OtherLetter, SC_Myanmar, LB_SA, EA_N,
    GB_OtherLetter, SC_Myanmar, LB_SA, EA_N, GB_OtherLetter, SC_Myanmar,
    LB_SA, EA_N, GB_OtherLetter, SC_Myanmar, LB_SA, EA_N, GB_OtherLetter,
    SC_Myanmar, LB_SA, EA_N, GB_OtherLetter, SC_Myanmar, LB_SA, EA_N,
    GB_OtherLetter, SC_Myanmar, LB_SA, EA_N, GB_OtherLetter, SC_Myanmar,
    LB_SA, EA_N, GB_OtherLetter, SC_Myanmar, LB_SA, EA_N, GB_OtherLetter,
    SC_Myanmar, LB_SA, EA_N, GB_OtherLetter, SC_Myanmar, LB_SA, EA_N,
    GB_OtherLetter, SC_Myanmar, LB_SA, EA_N, GB_OtherLetter, SC_Myanmar,
    LB_SA, EA_N, GB_OtherLetter, SC_Myanmar, LB_SA, EA_N, GB_OtherLetter,
    SC_Myanmar, LB_SA, EA_N, GB_OtherLetter, SC_Myanmar, LB_SA, EA_N,
    GB_OtherLetter, SC_Myanmar, LB_SA, EA_N, GB_OtherLetter, SC_Myanmar,
    LB_SA, EA_N, GB_OtherLetter, SC_Myanmar, LB_SA, EA_N, GB_OtherLetter,
    SC_Myanmar, LB_SA, EA_N, GB_OtherLetter, SC_Myanmar, LB_SA, EA_N,
    GB_OtherLetter, SC_Myanmar, LB_SA, EA_N, GB_OtherLetter, SC_Myanmar,
    LB_SA, EA_N, GB_OtherLetter, SC_Myanmar, LB_SA, EA_N, GB_OtherLetter,
    SC_Myanmar, LB_SA, EA_N, GB_OtherLetter, SC_Myanmar, LB_SA, EA_N,
    GB_OtherLetter, SC_Myanmar, LB_SA, EA_N, GB_OtherLetter, SC_Myanmar,
    LB_SA, EA_N, GB_Other, SC_Myanmar, LB_SA, EA_N, GB_Other, SC_Myanmar,
    LB_SA, EA_Z, GB_Extend, SC_Myanmar, LB_SA, EA_Z, GB_Extend, SC_Myanmar,
    LB_SA, EA_Z, GB_Extend, SC_Myanmar, LB_SA, EA_Z, GB_Extend, SC_Myanmar,
    LB_SA, EA_N, GB_SpacingMark, SC_Myanmar, LB_SA, EA_Z, GB_Extend,
    SC_Myanmar, LB_SA, EA_Z, GB_Extend, SC_Myanmar, LB_SA, EA_Z, GB_Extend,
    SC_Myanmar, LB_SA, EA_Z, GB_Extend, SC_Myanmar, LB_SA, EA_Z, GB_Extend,
    SC_Myanmar, LB_SA, EA_Z, GB_Extend, SC_Myanmar, LB_SA, EA_N, GB_Other,
    SC_Myanmar, LB_SA, EA_Z, GB_Virama, SC_Myanmar, LB_SA, EA_Z, GB_Virama,
    SC_Myanmar, LB_SA, EA_N, GB_SpacingMark, SC_Myanmar, LB_SA, EA_N,
    GB_SpacingMark, SC_Myanmar, LB_SA, EA_Z, GB_Extend, SC_Myanmar, LB_SA,
    EA_Z, GB_Extend, SC_Myanmar, LB_SA, EA_N, GB_OtherLetter, SC_Myanmar,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_SA, EA_N, GB_OtherLetter, SC_Myanmar, LB_SA, EA_N, GB_OtherLetter,
    SC_Myanmar, LB_SA, EA_N, GB_OtherLetter, SC_Myanmar, LB_SA, EA_N,
    GB_OtherLetter, SC_Myanmar, LB_SA, EA_N, GB_OtherLetter, SC_Myanmar,
    LB_SA, EA_N, GB_OtherLetter, SC_Myanmar, LB_SA, EA_N, GB_SpacingMark,
    SC_Myanmar, LB_SA, EA_N, GB_SpacingMark, SC_Myanmar, LB_SA, EA_Z,
    GB_Extend, SC_Myanmar, LB_SA, EA_Z, GB_Extend, SC_Myanmar, LB_SA, EA_N,
    GB_OtherLetter, SC_Myanmar, LB_SA, EA_N, GB_OtherLetter, SC_Myanmar,
    LB_SA, EA_N, GB_OtherLetter, SC_Myanmar, LB_SA, EA_N, GB_OtherLetter,
    SC_Myanmar, LB_SA, EA_Z, GB_Extend, SC_Myanmar, LB_SA, EA_Z, GB_Extend,
    SC_Myanmar, LB_SA, EA_N, GB_OtherLetter, SC_Myanmar, LB_SA, EA_N,
    GB_Other, SC_Myanmar, LB_SA, EA_N, GB_Other, SC_Myanmar, LB_SA, EA_N,
    GB_Other, SC_Myanmar, LB_SA, EA_N, GB_OtherLetter, SC_Myanmar, LB_SA,
    EA_N, GB_OtherLetter, SC_Myanmar, LB_SA, EA_N, GB_Other, SC_Myanmar,
    LB_SA, EA_N, GB_Other, SC_Myanmar, LB_SA, EA_N, GB_Other, SC_Myanmar,
    LB_SA, EA_N, GB_Other, SC_Myanmar, LB_SA, EA_N, GB_Other, SC_Myanmar,
    LB_SA, EA_N, GB_Other, SC_Myanmar, LB_SA, EA_N, GB_Other, SC_Myanmar,
    LB_SA, EA_N, GB_OtherLetter, SC_Myanmar, LB_SA, EA_N, GB_OtherLetter,
    SC_Myanmar, LB_SA, EA_N, GB_OtherLetter, SC_Myanmar, LB_SA, EA_Z,
    GB_Extend, SC_Myanmar, LB_SA, EA_Z, GB_Extend, SC_Myanmar, LB_SA, EA_Z,
    GB_Extend, SC_Myanmar, LB_SA, EA_Z, GB_Extend, SC_Myanmar, LB_SA, EA_N,
    GB_OtherLetter, SC_Myanmar, LB_SA, EA_N, GB_OtherLetter, SC_Myanmar,
    LB_SA, EA_N, GB_OtherLetter, SC_Myanmar, LB_SA, EA_N, GB_OtherLetter,
    SC_Myanmar, LB_SA, EA_N, GB_OtherLetter, SC_Myanmar, LB_SA, EA_N,
    GB_OtherLetter, SC_Myanmar, LB_SA, EA_N, GB_OtherLetter, SC_Myanmar,
    LB_SA, EA_N, GB_OtherLetter, SC_Myanmar, LB_SA, EA_N, GB_OtherLetter,
    SC_Myanmar, LB_SA, EA_N, GB_OtherLetter, SC_Myanmar, LB_SA, EA_N,
    GB_OtherLetter, SC_Myanmar, LB_SA, EA_Z, GB_Extend, SC_Myanmar, LB_SA,
    EA_N, GB_Other, SC_Myanmar, LB_SA, EA_N, GB_SpacingMark, SC_Myanmar,
    LB_SA, EA_Z, GB_Extend, SC_Myanmar, LB_SA, EA_Z, GB_Extend, SC_Myanmar,
    LB_SA, EA_N, GB_Other, SC_Myanmar, LB_SA, EA_N, GB_Other, SC_Myanmar,
    LB_SA, EA_N, GB_Other, SC_Myanmar, LB_SA, EA_N, GB_Other, SC_Myanmar,
    LB_SA, EA_N, GB_Other, SC_Myanmar, LB_SA, EA_N, GB_Other, SC_Myanmar,
    LB_SA, EA_Z, GB_Extend, SC_Myanmar, LB_SA, EA_N, GB_OtherLetter,
    SC_Myanmar, LB_SA, EA_N, GB_Other, SC_Myanmar, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_SA, EA_N, GB_Other,
    SC_Myanmar, LB_SA, EA_N, GB_Other, SC_Myanmar, LB_SA, EA_N, GB_Other,
    SC_Myanmar, LB_SA, EA_Z, GB_Extend, SC_Myanmar, LB_SA, EA_N, GB_Other,
    SC_Myanmar, LB_SA, EA_N, GB_Other, SC_Myanmar, LB_JL, EA_W, GB_L,
    SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown,
    LB_JL, EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown, LB_JL,
    EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L,
    SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown,
    LB_JL, EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown, LB_JL,
    EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L,
    SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown,
    LB_JL, EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown, LB_JL,
    EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L,
    SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown,
    LB_JL, EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown, LB_JL,
    EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L,
    SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown,
    LB_JL, EA_W, GB_L, SC_Unknown, LB_JV, EA_N, GB_V, SC_Unknown, LB_JV,
    EA_N, GB_V, SC_Unknown, LB_JV, EA_N, GB_V, SC_Unknown, LB_JV, EA_N, GB_V,
    SC_Unknown, LB_JV, EA_N, GB_V, SC_Unknown, LB_JV, EA_N, GB_V, SC_Unknown,
    LB_JV, EA_N, GB_V, SC_Unknown, LB_JV, EA_N, GB_V, SC_Unknown, LB_JV,
    EA_N, GB_V, SC_Unknown, LB_JV, EA_N, GB_V, SC_Unknown, LB_JV, EA_N, GB_V,
    SC_Unknown, LB_JV, EA_N, GB_V, SC_Unknown, LB_JV, EA_N, GB_V, SC_Unknown,
    LB_JV, EA_N, GB_V, SC_Unknown, LB_JV, EA_N, GB_V, SC_Unknown, LB_JV,
    EA_N, GB_V, SC_Unknown, LB_JV, EA_N, GB_V, SC_Unknown, LB_JV, EA_N, GB_V,
    SC_Unknown, LB_JV, EA_N, GB_V, SC_Unknown, LB_JV, EA_N, GB_V, SC_Unknown,
    LB_JV, EA_N, GB_V, SC_Unknown, LB_JV, EA_N, GB_V, SC_Unknown, LB_JV,
    EA_N, GB_V, SC_Unknown, LB_JV, EA_N, GB_V, SC_Unknown, LB_JV, EA_N, GB_V,
    SC_Unknown, LB_JV, EA_N, GB_V, SC_Unknown, LB_JV, EA_N, GB_V, SC_Unknown,
    LB_JV, EA_N, GB_V, SC_Unknown, LB_JV, EA_N, GB_V, SC_Unknown, LB_JV,
    EA_N, GB_V, SC_Unknown, LB_JV, EA_N, GB_V, SC_Unknown, LB_JV, EA_N, GB_V,
    SC_Unknown, LB_JV, EA_W, GB_V, SC_Unknown, LB_JV, EA_W, GB_V, SC_Unknown,
    LB_JV, EA_W, GB_V, SC_Unknown, LB_JV, EA_W, GB_V, SC_Unknown, LB_JV,
    EA_W, GB_V, SC_Unknown, LB_JT, EA_N, GB_T, SC_Unknown, LB_JT, EA_N, GB_T,
    SC_Unknown, LB_JT, EA_N, GB_T, SC_Unknown, LB_JT, EA_N, GB_T, SC_Unknown,
    LB_JT, EA_N, GB_T, SC_Unknown, LB_JT, EA_N, GB_T, SC_Unknown, LB_JT,
    EA_N, GB_T, SC_Unknown, LB_JT, EA_N, GB_T, SC_Unknown, LB_JT, EA_N, GB_T,
    SC_Unknown, LB_JT, EA_N, GB_T, SC_Unknown, LB_JT, EA_N, GB_T, SC_Unknown,
    LB_JT, EA_N, GB_T, SC_Unknown, LB_JT, EA_N, GB_T, SC_Unknown, LB_JT,
    EA_N, GB_T, SC_Unknown, LB_JT, EA_N, GB_T, SC_Unknown, LB_JT, EA_N, GB_T,
    SC_Unknown, LB_JT, EA_N, GB_T, SC_Unknown, LB_JT, EA_N, GB_T, SC_Unknown,
    LB_JT, EA_N, GB_T, SC_Unknown, LB_JT, EA_N, GB_T, SC_Unknown, LB_JT,
    EA_N, GB_T, SC_Unknown, LB_JT, EA_N, GB_T, SC_Unknown, LB_JT, EA_N, GB_T,
    SC_Unknown, LB_JT, EA_N, GB_T, SC_Unknown, LB_JT, EA_N, GB_T, SC_Unknown,
    LB_JT, EA_N, GB_T, SC_Unknown, LB_JT, EA_N, GB_T, SC_Unknown, LB_JT,
    EA_N, GB_T, SC_Unknown, LB_JT, EA_N, GB_T, SC_Unknown, LB_JT, EA_N, GB_T,
    SC_Unknown, LB_JT, EA_N, GB_T, SC_Unknown, LB_JT, EA_N, GB_T, SC_Unknown,
    LB_JT, EA_W, GB_T, SC_Unknown, LB_JT, EA_W, GB_T, SC_Unknown, LB_JT,
    EA_W, GB_T, SC_Unknown, LB_JT, EA_W, GB_T, SC_Unknown, LB_JT, EA_W, GB_T,
    SC_Unknown, LB_JT, EA_W, GB_T, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_OP, EA_N, GB_Other,
    SC_Unknown, LB_CL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other,
    SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Virama,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Virama,
    SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_SA, EA_N, GB_OtherLetter, SC_Khmer, LB_SA, EA_N,
    GB_OtherLetter, SC_Khmer, LB_SA, EA_N, GB_OtherLetter, SC_Khmer, LB_SA,
    EA_N, GB_OtherLetter, SC_Khmer, LB_SA, EA_N, GB_OtherLetter, SC_Khmer,
    LB_SA, EA_N, GB_OtherLetter, SC_Khmer, LB_SA, EA_N, GB_OtherLetter,
    SC_Khmer, LB_SA, EA_N, GB_OtherLetter, SC_Khmer, LB_SA, EA_N,
    GB_OtherLetter, SC_Khmer, LB_SA, EA_N, GB_OtherLetter, SC_Khmer, LB_SA,
    EA_N, GB_OtherLetter, SC_Khmer, LB_SA, EA_N, GB_OtherLetter, SC_Khmer,
    LB_SA, EA_N, GB_OtherLetter, SC_Khmer, LB_SA, EA_N, GB_OtherLetter,
    SC_Khmer, LB_SA, EA_N, GB_OtherLetter, SC_Khmer, LB_SA, EA_N,
    GB_OtherLetter, SC_Khmer, LB_SA, EA_N, GB_OtherLetter, SC_Khmer, LB_SA,
    EA_N, GB_OtherLetter, SC_Khmer, LB_SA, EA_N, GB_OtherLetter, SC_Khmer,
    LB_SA, EA_N, GB_OtherLetter, SC_Khmer, LB_SA, EA_N, GB_OtherLetter,
    SC_Khmer, LB_SA, EA_N, GB_OtherLetter, SC_Khmer, LB_SA, EA_N,
    GB_OtherLetter, SC_Khmer, LB_SA, EA_N, GB_OtherLetter, SC_Khmer, LB_SA,
    EA_N, GB_OtherLetter, SC_Khmer, LB_SA, EA_N, GB_OtherLetter, SC_Khmer,
    LB_SA, EA_N, GB_OtherLetter, SC_Khmer, LB_SA, EA_N, GB_OtherLetter,
    SC_Khmer, LB_SA, EA_N, GB_OtherLetter, SC_Khmer, LB_SA, EA_N,
    GB_OtherLetter, SC_Khmer, LB_SA, EA_N, GB_OtherLetter, SC_Khmer, LB_SA,
    EA_N, GB_OtherLetter, SC_Khmer, LB_SA, EA_Z, GB_Extend, SC_Khmer, LB_SA,
    EA_Z, GB_Extend, SC_Khmer, LB_SA, EA_N, GB_SpacingMark, SC_Khmer, LB_SA,
    EA_Z, GB_Extend, SC_Khmer, LB_SA, EA_Z, GB_Extend, SC_Khmer, LB_SA, EA_Z,
    GB_Extend, SC_Khmer, LB_SA, EA_Z, GB_Extend, SC_Khmer, LB_SA, EA_Z,
    GB_Extend, SC_Khmer, LB_SA, EA_Z, GB_Extend, SC_Khmer, LB_SA, EA_Z,
    GB_Extend, SC_Khmer, LB_SA, EA_N, GB_SpacingMark, SC_Khmer, LB_SA, EA_N,
    GB_SpacingMark, SC_Khmer, LB_SA, EA_N, GB_SpacingMark, SC_Khmer, LB_SA,
    EA_N, GB_SpacingMark, SC_Khmer, LB_SA, EA_N, GB_SpacingMark, SC_Khmer,
    LB_SA, EA_N, GB_SpacingMark, SC_Khmer, LB_SA, EA_Z, GB_Extend, SC_Khmer,
    LB_SA, EA_N, GB_SpacingMark, SC_Khmer, LB_SA, EA_N, GB_SpacingMark,
    SC_Khmer, LB_SA, EA_Z, GB_Extend, SC_Khmer, LB_SA, EA_Z, GB_Extend,
    SC_Khmer, LB_SA, EA_Z, GB_Extend, SC_Khmer, LB_SA, EA_Z, GB_Extend,
    SC_Khmer, LB_SA, EA_Z, GB_Extend, SC_Khmer, LB_SA, EA_Z, GB_Extend,
    SC_Khmer, LB_SA, EA_Z, GB_Extend, SC_Khmer, LB_SA, EA_Z, GB_Extend,
    SC_Khmer, LB_SA, EA_Z, GB_Extend, SC_Khmer, LB_SA, EA_Z, GB_Virama,
    SC_Khmer, LB_SA, EA_Z, GB_Extend, SC_Khmer, LB_BA, EA_N, GB_Other,
    SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_NS, EA_N, GB_Other,
    SC_Unknown, LB_SA, EA_N, GB_OtherLetter, SC_Khmer, LB_BA, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other,
    SC_Unknown, LB_PR, EA_N, GB_Other, SC_Unknown, LB_SA, EA_N,
    GB_OtherLetter, SC_Khmer, LB_SA, EA_Z, GB_Extend, SC_Khmer, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_EX, EA_N,
    GB_Other, SC_Unknown, LB_EX, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N,
    GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BB, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_EX, EA_N,
    GB_Other, SC_Unknown, LB_EX, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_GL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N,
    GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N,
    GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N,
    GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N,
    GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N,
    GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM,
    EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_EX, EA_N, GB_Other,
    SC_Unknown, LB_EX, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_SA, EA_N, GB_Other,
    SC_Tai_Le, LB_SA, EA_N, GB_Other, SC_Tai_Le, LB_SA, EA_N, GB_Other,
    SC_Tai_Le, LB_SA, EA_N, GB_Other, SC_Tai_Le, LB_SA, EA_N, GB_Other,
    SC_Tai_Le, LB_SA, EA_N, GB_Other, SC_Tai_Le, LB_SA, EA_N, GB_Other,
    SC_Tai_Le, LB_SA, EA_N, GB_Other, SC_Tai_Le, LB_SA, EA_N, GB_Other,
    SC_Tai_Le, LB_SA, EA_N, GB_Other, SC_Tai_Le, LB_SA, EA_N, GB_Other,
    SC_Tai_Le, LB_SA, EA_N, GB_Other, SC_Tai_Le, LB_SA, EA_N, GB_Other,
    SC_Tai_Le, LB_SA, EA_N, GB_Other, SC_Tai_Le, LB_SA, EA_N, GB_Other,
    SC_Tai_Le, LB_SA, EA_N, GB_Other, SC_Tai_Le, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_SA, EA_N, GB_Other,
    SC_Tai_Le, LB_SA, EA_N, GB_Other, SC_Tai_Le, LB_SA, EA_N, GB_Other,
    SC_Tai_Le, LB_SA, EA_N, GB_Other, SC_Tai_Le, LB_SA, EA_N, GB_Other,
    SC_Tai_Le, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_SA, EA_N, GB_Other,
    SC_New_Tai_Lue, LB_SA, EA_N, GB_Other, SC_New_Tai_Lue, LB_SA, EA_N,
    GB_Other, SC_New_Tai_Lue, LB_SA, EA_N, GB_Other, SC_New_Tai_Lue, LB_SA,
    EA_N, GB_Other, SC_New_Tai_Lue, LB_SA, EA_N, GB_Other, SC_New_Tai_Lue,
    LB_SA, EA_N, GB_Other, SC_New_Tai_Lue, LB_SA, EA_N, GB_Other,
    SC_New_Tai_Lue, LB_SA, EA_N, GB_Other, SC_New_Tai_Lue, LB_SA, EA_N,
    GB_Other, SC_New_Tai_Lue, LB_SA, EA_N, GB_Other, SC_New_Tai_Lue, LB_SA,
    EA_N, GB_Other, SC_New_Tai_Lue, LB_SA, EA_N, GB_Other, SC_New_Tai_Lue,
    LB_SA, EA_N, GB_Other, SC_New_Tai_Lue, LB_SA, EA_N, GB_Other,
    SC_New_Tai_Lue, LB_SA, EA_N, GB_Other, SC_New_Tai_Lue, LB_SA, EA_N,
    GB_Other, SC_New_Tai_Lue, LB_SA, EA_N, GB_Other, SC_New_Tai_Lue, LB_SA,
    EA_N, GB_Other, SC_New_Tai_Lue, LB_SA, EA_N, GB_Other, SC_New_Tai_Lue,
    LB_SA, EA_N, GB_Other, SC_New_Tai_Lue, LB_SA, EA_N, GB_Other,
    SC_New_Tai_Lue, LB_SA, EA_N, GB_Other, SC_New_Tai_Lue, LB_SA, EA_N,
    GB_Other, SC_New_Tai_Lue, LB_SA, EA_N, GB_Other, SC_New_Tai_Lue, LB_SA,
    EA_N, GB_Other, SC_New_Tai_Lue, LB_SA, EA_N, GB_Other, SC_New_Tai_Lue,
    LB_SA, EA_N, GB_Other, SC_New_Tai_Lue, LB_SA, EA_N, GB_Other,
    SC_New_Tai_Lue, LB_SA, EA_N, GB_Other, SC_New_Tai_Lue, LB_SA, EA_N,
    GB_Other, SC_New_Tai_Lue, LB_SA, EA_N, GB_Other, SC_New_Tai_Lue, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_SA,
    EA_N, GB_Other, SC_New_Tai_Lue, LB_SA, EA_N, GB_Other, SC_New_Tai_Lue,
    LB_SA, EA_N, GB_Other, SC_New_Tai_Lue, LB_SA, EA_N, GB_Other,
    SC_New_Tai_Lue, LB_SA, EA_N, GB_Other, SC_New_Tai_Lue, LB_SA, EA_N,
    GB_SpacingMark, SC_New_Tai_Lue, LB_SA, EA_N, GB_SpacingMark,
    SC_New_Tai_Lue, LB_SA, EA_N, GB_SpacingMark, SC_New_Tai_Lue, LB_SA, EA_N,
    GB_Other, SC_New_Tai_Lue, LB_SA, EA_N, GB_Other, SC_New_Tai_Lue, LB_SA,
    EA_N, GB_SpacingMark, SC_New_Tai_Lue, LB_SA, EA_N, GB_Other,
    SC_New_Tai_Lue, LB_SA, EA_N, GB_Other, SC_New_Tai_Lue, LB_SA, EA_N,
    GB_Other, SC_New_Tai_Lue, LB_SA, EA_N, GB_Other, SC_New_Tai_Lue, LB_SA,
    EA_N, GB_Other, SC_New_Tai_Lue, LB_SA, EA_N, GB_Other, SC_New_Tai_Lue,
    LB_SA, EA_N, GB_Other, SC_New_Tai_Lue, LB_SA, EA_N, GB_Other,
    SC_New_Tai_Lue, LB_SA, EA_N, GB_Other, SC_New_Tai_Lue, LB_SA, EA_N,
    GB_Other, SC_New_Tai_Lue, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N,
    GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N,
    GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N,
    GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N,
    GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N,
    GB_Other, SC_Unknown, LB_SA, EA_N, GB_Other, SC_New_Tai_Lue, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_SA, EA_N, GB_Other, SC_New_Tai_Lue, LB_SA, EA_N,
    GB_Other, SC_New_Tai_Lue, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_SA, EA_N,
    GB_OtherLetter, SC_Tai_Tham, LB_SA, EA_N, GB_OtherLetter, SC_Tai_Tham,
    LB_SA, EA_N, GB_OtherLetter, SC_Tai_Tham, LB_SA, EA_N, GB_OtherLetter,
    SC_Tai_Tham, LB_SA, EA_N, GB_OtherLetter, SC_Tai_Tham, LB_SA, EA_N,
    GB_OtherLetter, SC_Tai_Tham, LB_SA, EA_N, GB_OtherLetter, SC_Tai_Tham,
    LB_SA, EA_N, GB_OtherLetter, SC_Tai_Tham, LB_SA, EA_N, GB_OtherLetter,
    SC_Tai_Tham, LB_SA, EA_N, GB_OtherLetter, SC_Tai_Tham, LB_SA, EA_N,
    GB_OtherLetter, SC_Tai_Tham, LB_SA, EA_N, GB_OtherLetter, SC_Tai_Tham,
    LB_SA, EA_N, GB_OtherLetter, SC_Tai_Tham, LB_SA, EA_N, GB_OtherLetter,
    SC_Tai_Tham, LB_SA, EA_N, GB_OtherLetter, SC_Tai_Tham, LB_SA, EA_N,
    GB_OtherLetter, SC_Tai_Tham, LB_SA, EA_N, GB_OtherLetter, SC_Tai_Tham,
    LB_SA, EA_N, GB_OtherLetter, SC_Tai_Tham, LB_SA, EA_N, GB_OtherLetter,
    SC_Tai_Tham, LB_SA, EA_N, GB_OtherLetter, SC_Tai_Tham, LB_SA, EA_N,
    GB_OtherLetter, SC_Tai_Tham, LB_SA, EA_N, GB_OtherLetter, SC_Tai_Tham,
    LB_SA, EA_N, GB_OtherLetter, SC_Tai_Tham, LB_SA, EA_N, GB_OtherLetter,
    SC_Tai_Tham, LB_SA, EA_N, GB_OtherLetter, SC_Tai_Tham, LB_SA, EA_N,
    GB_OtherLetter, SC_Tai_Tham, LB_SA, EA_N, GB_OtherLetter, SC_Tai_Tham,
    LB_SA, EA_N, GB_OtherLetter, SC_Tai_Tham, LB_SA, EA_N, GB_OtherLetter,
    SC_Tai_Tham, LB_SA, EA_N, GB_OtherLetter, SC_Tai_Tham, LB_SA, EA_N,
    GB_OtherLetter, SC_Tai_Tham, LB_SA, EA_N, GB_OtherLetter, SC_Tai_Tham,
    LB_SA, EA_N, GB_SpacingMark, SC_Tai_Tham, LB_SA, EA_Z, GB_Extend,
    SC_Tai_Tham, LB_SA, EA_N, GB_SpacingMark, SC_Tai_Tham, LB_SA, EA_Z,
    GB_Extend, SC_Tai_Tham, LB_SA, EA_Z, GB_Extend, SC_Tai_Tham, LB_SA, EA_Z,
    GB_Extend, SC_Tai_Tham, LB_SA, EA_Z, GB_Extend, SC_Tai_Tham, LB_SA, EA_Z,
    GB_Extend, SC_Tai_Tham, LB_SA, EA_Z, GB_Extend, SC_Tai_Tham, LB_SA, EA_Z,
    GB_Extend, SC_Tai_Tham, LB_AL, EA_N, GB_Other, SC_Unknown, LB_SA, EA_Z,
    GB_Virama, SC_Tai_Tham, LB_SA, EA_N, GB_Other, SC_Tai_Tham, LB_SA, EA_Z,
    GB_Extend, SC_Tai_Tham, LB_SA, EA_N, GB_Other, SC_Tai_Tham, LB_SA, EA_N,
    GB_Other, SC_Tai_Tham, LB_SA, EA_Z, GB_Extend, SC_Tai_Tham, LB_SA, EA_Z,
    GB_Extend, SC_Tai_Tham, LB_SA, EA_Z, GB_Extend, SC_Tai_Tham, LB_SA, EA_Z,
    GB_Extend, SC_Tai_Tham, LB_SA, EA_Z, GB_Extend, SC_Tai_Tham, LB_SA, EA_Z,
    GB_Extend, SC_Tai_Tham, LB_SA, EA_Z, GB_Extend, SC_Tai_Tham, LB_SA, EA_Z,
    GB_Extend, SC_Tai_Tham, LB_SA, EA_N, GB_SpacingMark, SC_Tai_Tham, LB_SA,
    EA_N, GB_SpacingMark, SC_Tai_Tham, LB_SA, EA_N, GB_SpacingMark,
    SC_Tai_Tham, LB_SA, EA_N, GB_SpacingMark, SC_Tai_Tham, LB_SA, EA_N,
    GB_SpacingMark, SC_Tai_Tham, LB_SA, EA_N, GB_SpacingMark, SC_Tai_Tham,
    LB_SA, EA_Z, GB_Extend, SC_Tai_Tham, LB_SA, EA_Z, GB_Extend, SC_Tai_Tham,
    LB_SA, EA_Z, GB_Extend, SC_Tai_Tham, LB_SA, EA_Z, GB_Extend, SC_Tai_Tham,
    LB_SA, EA_Z, GB_Extend, SC_Tai_Tham, LB_SA, EA_Z, GB_Extend, SC_Tai_Tham,
    LB_SA, EA_Z, GB_Extend, SC_Tai_Tham, LB_SA, EA_Z, GB_Extend, SC_Tai_Tham,
    LB_SA, EA_Z, GB_Extend, SC_Tai_Tham, LB_SA, EA_Z, GB_Extend, SC_Tai_Tham,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_SA, EA_N, GB_Other, SC_Tai_Tham,
    LB_SA, EA_N, GB_Other, SC_Tai_Tham, LB_SA, EA_N, GB_Other, SC_Tai_Tham,
    LB_SA, EA_N, GB_Other, SC_Tai_Tham, LB_SA, EA_N, GB_Other, SC_Tai_Tham,
    LB_SA, EA_N, GB_Other, SC_Tai_Tham, LB_SA, EA_N, GB_Other, SC_Tai_Tham,
    LB_SA, EA_N, GB_OtherLetter, SC_Tai_Tham, LB_SA, EA_N, GB_Other,
    SC_Tai_Tham, LB_SA, EA_N, GB_Other, SC_Tai_Tham, LB_SA, EA_N, GB_Other,
    SC_Tai_Tham, LB_SA, EA_N, GB_Other, SC_Tai_Tham, LB_SA, EA_N, GB_Other,
    SC_Tai_Tham, LB_SA, EA_N, GB_Other, SC_Tai_Tham, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_Virama, SC_Unknown, LB_AL,
    EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other,
    SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_N, GB_Virama,
    SC_Unknown, LB_CM, EA_Z, GB_Virama, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_N, GB_Virama, SC_Unknown, LB_CM, EA_N, GB_Virama, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown,
    LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown,
    LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_BB, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown,
    LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown,
    LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown,
    LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown,
    LB_GL, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown,
    LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown,
    LB_ZW, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_BA, EA_A, GB_Other, SC_Unknown,
    LB_GL, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown,
    LB_BA, EA_A, GB_Other, SC_Unknown, LB_B2, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_QU, EA_A, GB_Other, SC_Unknown,
    LB_QU, EA_A, GB_Other, SC_Unknown, LB_OP, EA_N, GB_Other, SC_Unknown,
    LB_QU, EA_N, GB_Other, SC_Unknown, LB_QU, EA_A, GB_Other, SC_Unknown,
    LB_QU, EA_A, GB_Other, SC_Unknown, LB_OP, EA_N, GB_Other, SC_Unknown,
    LB_QU, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_IN, EA_A, GB_Other, SC_Unknown,
    LB_IN, EA_A, GB_Other, SC_Unknown, LB_IN, EA_A, GB_Other, SC_Unknown,
    LB_BA, EA_A, GB_Other, SC_Unknown, LB_BK, EA_Z, GB_Control, SC_Unknown,
    LB_BK, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_GL, EA_N, GB_Other, SC_Unknown, LB_PO, EA_A, GB_Other, SC_Unknown,
    LB_PO, EA_N, GB_Other, SC_Unknown, LB_PO, EA_A, GB_Other, SC_Unknown,
    LB_PO, EA_A, GB_Other, SC_Unknown, LB_PO, EA_N, GB_Other, SC_Unknown,
    LB_PO, EA_A, GB_Other, SC_Unknown, LB_PO, EA_N, GB_Other, SC_Unknown,
    LB_PO, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_QU, EA_N, GB_Other, SC_Unknown, LB_QU, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_NS, EA_N, GB_Other, SC_Unknown,
    LB_NS, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_IS, EA_N, GB_Other, SC_Unknown, LB_OP, EA_N, GB_Other, SC_Unknown,
    LB_CL, EA_N, GB_Other, SC_Unknown, LB_NS, EA_N, GB_Other, SC_Unknown,
    LB_NS, EA_N, GB_Other, SC_Unknown, LB_NS, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_BA, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown,
    LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown,
    LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown,
    LB_WJ, EA_Z, GB_Control, SC_Unknown, LB_AL, EA_Z, GB_Control, SC_Unknown,
    LB_AL, EA_Z, GB_Control, SC_Unknown, LB_AL, EA_Z, GB_Control, SC_Unknown,
    LB_AL, EA_Z, GB_Control, SC_Unknown, LB_AL, EA_N, GB_Control, SC_Unknown,
    LB_AL, EA_N, GB_Control, SC_Unknown, LB_AL, EA_N, GB_Control, SC_Unknown,
    LB_AL, EA_N, GB_Control, SC_Unknown, LB_AL, EA_N, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_OP, EA_N, GB_Other, SC_Unknown,
    LB_CL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_OP, EA_N, GB_Other, SC_Unknown,
    LB_CL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_PR, EA_N, GB_Other, SC_Unknown, LB_PR, EA_N, GB_Other, SC_Unknown,
    LB_PR, EA_N, GB_Other, SC_Unknown, LB_PR, EA_N, GB_Other, SC_Unknown,
    LB_PR, EA_N, GB_Other, SC_Unknown, LB_PR, EA_N, GB_Other, SC_Unknown,
    LB_PR, EA_N, GB_Other, SC_Unknown, LB_PO, EA_N, GB_Other, SC_Unknown,
    LB_PR, EA_N, GB_Other, SC_Unknown, LB_PR, EA_H, GB_Other, SC_Unknown,
    LB_PR, EA_N, GB_Other, SC_Unknown, LB_PR, EA_N, GB_Other, SC_Unknown,
    LB_PR, EA_A, GB_Other, SC_Unknown, LB_PR, EA_N, GB_Other, SC_Unknown,
    LB_PR, EA_N, GB_Other, SC_Unknown, LB_PR, EA_N, GB_Other, SC_Unknown,
    LB_PR, EA_N, GB_Other, SC_Unknown, LB_PR, EA_N, GB_Other, SC_Unknown,
    LB_PR, EA_N, GB_Other, SC_Unknown, LB_PR, EA_N, GB_Other, SC_Unknown,
    LB_PR, EA_N, GB_Other, SC_Unknown, LB_PR, EA_N, GB_Other, SC_Unknown,
    LB_PO, EA_N, GB_Other, SC_Unknown, LB_PR, EA_N, GB_Other, SC_Unknown,
    LB_PR, EA_N, GB_Other, SC_Unknown, LB_PR, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_PO, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_PO, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_PR, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_PR, EA_N, GB_Other, SC_Unknown, LB_PR, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_OP, EA_W, GB_Other, SC_Unknown,
    LB_CL, EA_W, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_QU, EA_N, GB_Other, SC_Unknown, LB_QU, EA_N, GB_Other, SC_Unknown,
    LB_QU, EA_N, GB_Other, SC_Unknown, LB_QU, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_EX, EA_N, GB_Other, SC_Unknown, LB_EX, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_OP, EA_N, GB_Other, SC_Unknown, LB_CL, EA_N, GB_Other, SC_Unknown,
    LB_OP, EA_N, GB_Other, SC_Unknown, LB_CL, EA_N, GB_Other, SC_Unknown,
    LB_OP, EA_N, GB_Other, SC_Unknown, LB_CL, EA_N, GB_Other, SC_Unknown,
    LB_OP, EA_N, GB_Other, SC_Unknown, LB_CL, EA_N, GB_Other, SC_Unknown,
    LB_OP, EA_N, GB_Other, SC_Unknown, LB_CL, EA_N, GB_Other, SC_Unknown,
    LB_OP, EA_N, GB_Other, SC_Unknown, LB_CL, EA_N, GB_Other, SC_Unknown,
    LB_OP, EA_N, GB_Other, SC_Unknown, LB_CL, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_N, GB_Other, SC_Unknown, LB_AI, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_N, GB_Other, SC_Unknown, LB_AI, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_N, GB_Other, SC_Unknown, LB_AI, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_N, GB_Other, SC_Unknown, LB_AI, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_N, GB_Other, SC_Unknown, LB_AI, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_N, GB_Other, SC_Unknown, LB_AI, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_N, GB_Other, SC_Unknown, LB_AI, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_N, GB_Other, SC_Unknown, LB_AI, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_N, GB_Other, SC_Unknown, LB_AI, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_N, GB_Other, SC_Unknown, LB_AI, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_OP, EA_N, GB_Other, SC_Unknown, LB_CL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_OP, EA_Na, GB_Other, SC_Unknown,
    LB_CL, EA_Na, GB_Other, SC_Unknown, LB_OP, EA_Na, GB_Other, SC_Unknown,
    LB_CL, EA_Na, GB_Other, SC_Unknown, LB_OP, EA_Na, GB_Other, SC_Unknown,
    LB_CL, EA_Na, GB_Other, SC_Unknown, LB_OP, EA_Na, GB_Other, SC_Unknown,
    LB_CL, EA_Na, GB_Other, SC_Unknown, LB_OP, EA_N, GB_Other, SC_Unknown,
    LB_CL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_OP, EA_N, GB_Other, SC_Unknown,
    LB_CL, EA_N, GB_Other, SC_Unknown, LB_OP, EA_Na, GB_Other, SC_Unknown,
    LB_CL, EA_Na, GB_Other, SC_Unknown, LB_OP, EA_N, GB_Other, SC_Unknown,
    LB_CL, EA_N, GB_Other, SC_Unknown, LB_OP, EA_N, GB_Other, SC_Unknown,
    LB_CL, EA_N, GB_Other, SC_Unknown, LB_OP, EA_N, GB_Other, SC_Unknown,
    LB_CL, EA_N, GB_Other, SC_Unknown, LB_OP, EA_N, GB_Other, SC_Unknown,
    LB_CL, EA_N, GB_Other, SC_Unknown, LB_OP, EA_N, GB_Other, SC_Unknown,
    LB_CL, EA_N, GB_Other, SC_Unknown, LB_OP, EA_N, GB_Other, SC_Unknown,
    LB_CL, EA_N, GB_Other, SC_Unknown, LB_OP, EA_N, GB_Other, SC_Unknown,
    LB_CL, EA_N, GB_Other, SC_Unknown, LB_OP, EA_N, GB_Other, SC_Unknown,
    LB_CL, EA_N, GB_Other, SC_Unknown, LB_OP, EA_N, GB_Other, SC_Unknown,
    LB_CL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_OP, EA_N, GB_Other, SC_Unknown,
    LB_CL, EA_N, GB_Other, SC_Unknown, LB_OP, EA_N, GB_Other, SC_Unknown,
    LB_CL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_OP, EA_N, GB_Other, SC_Unknown,
    LB_CL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_EX, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown,
    LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_EX, EA_N, GB_Other, SC_Unknown,
    LB_BA, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM,
    EA_Z, GB_Virama, SC_Unknown, LB_QU, EA_N, GB_Other, SC_Unknown, LB_QU,
    EA_N, GB_Other, SC_Unknown, LB_QU, EA_N, GB_Other, SC_Unknown, LB_QU,
    EA_N, GB_Other, SC_Unknown, LB_QU, EA_N, GB_Other, SC_Unknown, LB_QU,
    EA_N, GB_Other, SC_Unknown, LB_QU, EA_N, GB_Other, SC_Unknown, LB_QU,
    EA_N, GB_Other, SC_Unknown, LB_QU, EA_N, GB_Other, SC_Unknown, LB_QU,
    EA_N, GB_Other, SC_Unknown, LB_QU, EA_N, GB_Other, SC_Unknown, LB_QU,
    EA_N, GB_Other, SC_Unknown, LB_QU, EA_N, GB_Other, SC_Unknown, LB_QU,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_OP, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_QU, EA_N, GB_Other, SC_Unknown, LB_QU,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_QU, EA_N, GB_Other, SC_Unknown, LB_QU,
    EA_N, GB_Other, SC_Unknown, LB_OP, EA_N, GB_Other, SC_Unknown, LB_CL,
    EA_N, GB_Other, SC_Unknown, LB_OP, EA_N, GB_Other, SC_Unknown, LB_CL,
    EA_N, GB_Other, SC_Unknown, LB_OP, EA_N, GB_Other, SC_Unknown, LB_CL,
    EA_N, GB_Other, SC_Unknown, LB_OP, EA_N, GB_Other, SC_Unknown, LB_CL,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_EX, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_B2, EA_N, GB_Other, SC_Unknown, LB_B2,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_ID,
    EA_F, GB_Other, SC_Unknown, LB_CL, EA_W, GB_Other, SC_Unknown, LB_CL,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_NS, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_OP,
    EA_W, GB_Other, SC_Unknown, LB_CL, EA_W, GB_Other, SC_Unknown, LB_OP,
    EA_W, GB_Other, SC_Unknown, LB_CL, EA_W, GB_Other, SC_Unknown, LB_OP,
    EA_W, GB_Other, SC_Unknown, LB_CL, EA_W, GB_Other, SC_Unknown, LB_OP,
    EA_W, GB_Other, SC_Unknown, LB_CL, EA_W, GB_Other, SC_Unknown, LB_OP,
    EA_W, GB_Other, SC_Unknown, LB_CL, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_OP,
    EA_W, GB_Other, SC_Unknown, LB_CL, EA_W, GB_Other, SC_Unknown, LB_OP,
    EA_W, GB_Other, SC_Unknown, LB_CL, EA_W, GB_Other, SC_Unknown, LB_OP,
    EA_W, GB_Other, SC_Unknown, LB_CL, EA_W, GB_Other, SC_Unknown, LB_OP,
    EA_W, GB_Other, SC_Unknown, LB_CL, EA_W, GB_Other, SC_Unknown, LB_NS,
    EA_W, GB_Other, SC_Unknown, LB_OP, EA_W, GB_Other, SC_Unknown, LB_CL,
    EA_W, GB_Other, SC_Unknown, LB_CL, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_W, GB_Extend, SC_Unknown, LB_CM, EA_W, GB_Extend, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_NS, EA_W, GB_Other, SC_Unknown, LB_NS,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_CJ, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_CJ, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_CJ, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_CJ, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_CJ, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_CJ,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_CJ, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_CJ, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_CJ, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_CJ,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_CJ, EA_W, GB_Other, SC_Unknown, LB_CJ,
    EA_W, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_NS, EA_W, GB_Other, SC_Unknown, LB_NS,
    EA_W, GB_Other, SC_Unknown, LB_NS, EA_W, GB_Other, SC_Unknown, LB_NS,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_NS,
    EA_W, GB_Other, SC_Unknown, LB_CJ, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_CJ, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_CJ, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_CJ, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_CJ, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_CJ,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_CJ,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_CJ,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_CJ, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_CJ,
    EA_W, GB_Other, SC_Unknown, LB_CJ, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_NS,
    EA_W, GB_Other, SC_Unknown, LB_CJ, EA_W, GB_Other, SC_Unknown, LB_NS,
    EA_W, GB_Other, SC_Unknown, LB_NS, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_CJ, EA_W, GB_Other, SC_Unknown, LB_CJ,
    EA_W, GB_Other, SC_Unknown, LB_CJ, EA_W, GB_Other, SC_Unknown, LB_CJ,
    EA_W, GB_Other, SC_Unknown, LB_CJ, EA_W, GB_Other, SC_Unknown, LB_CJ,
    EA_W, GB_Other, SC_Unknown, LB_CJ, EA_W, GB_Other, SC_Unknown, LB_CJ,
    EA_W, GB_Other, SC_Unknown, LB_CJ, EA_W, GB_Other, SC_Unknown, LB_CJ,
    EA_W, GB_Other, SC_Unknown, LB_CJ, EA_W, GB_Other, SC_Unknown, LB_CJ,
    EA_W, GB_Other, SC_Unknown, LB_CJ, EA_W, GB_Other, SC_Unknown, LB_CJ,
    EA_W, GB_Other, SC_Unknown, LB_CJ, EA_W, GB_Other, SC_Unknown, LB_CJ,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_NS,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_EX, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_CM, EA_Z, GB_Virama, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_PO, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_BB,
    EA_N, GB_Other, SC_Unknown, LB_BB, EA_N, GB_Other, SC_Unknown, LB_EX,
    EA_N, GB_Other, SC_Unknown, LB_EX, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM,
    EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_Z, GB_Virama, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM,
    EA_N, GB_Virama, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown, LB_JL, EA_W,
    GB_L, SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L,
    SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown,
    LB_JL, EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown, LB_JL,
    EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L,
    SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown,
    LB_JL, EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown, LB_JL,
    EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L,
    SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown,
    LB_JL, EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown, LB_JL,
    EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L,
    SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown,
    LB_JL, EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM,
    EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM,
    EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_N, GB_Virama, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other,
    SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_N, GB_SpacingMark, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown,
    LB_BA, EA_N, GB_Other, SC_Unknown, LB_SA, EA_N, GB_OtherLetter,
    SC_Myanmar, LB_SA, EA_N, GB_OtherLetter, SC_Myanmar, LB_SA, EA_N,
    GB_OtherLetter, SC_Myanmar, LB_SA, EA_N, GB_OtherLetter, SC_Myanmar,
    LB_SA, EA_N, GB_OtherLetter, SC_Myanmar, LB_SA, EA_N, GB_OtherLetter,
    SC_Myanmar, LB_SA, EA_N, GB_OtherLetter, SC_Myanmar, LB_SA, EA_N,
    GB_OtherLetter, SC_Myanmar, LB_SA, EA_N, GB_OtherLetter, SC_Myanmar,
    LB_SA, EA_N, GB_OtherLetter, SC_Myanmar, LB_SA, EA_N, GB_OtherLetter,
    SC_Myanmar, LB_SA, EA_N, GB_OtherLetter, SC_Myanmar, LB_SA, EA_N,
    GB_OtherLetter, SC_Myanmar, LB_SA, EA_N, GB_OtherLetter, SC_Myanmar,
    LB_SA, EA_N, GB_OtherLetter, SC_Myanmar, LB_SA, EA_N, GB_OtherLetter,
    SC_Myanmar, LB_SA, EA_N, GB_OtherLetter, SC_Myanmar, LB_SA, EA_N,
    GB_OtherLetter, SC_Myanmar, LB_SA, EA_N, GB_OtherLetter, SC_Myanmar,
    LB_SA, EA_N, GB_OtherLetter, SC_Myanmar, LB_SA, EA_N, GB_OtherLetter,
    SC_Myanmar, LB_SA, EA_N, GB_OtherLetter, SC_Myanmar, LB_SA, EA_N,
    GB_OtherLetter, SC_Myanmar, LB_SA, EA_N, GB_Other, SC_Myanmar, LB_SA,
    EA_N, GB_Other, SC_Myanmar, LB_SA, EA_N, GB_Other, SC_Myanmar, LB_SA,
    EA_N, GB_OtherLetter, SC_Myanmar, LB_SA, EA_N, GB_Other, SC_Myanmar,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_SA, EA_N, GB_Other, SC_Tai_Viet, LB_SA, EA_N, GB_Other, SC_Tai_Viet,
    LB_SA, EA_N, GB_Other, SC_Tai_Viet, LB_SA, EA_N, GB_Other, SC_Tai_Viet,
    LB_SA, EA_N, GB_Other, SC_Tai_Viet, LB_SA, EA_N, GB_Other, SC_Tai_Viet,
    LB_SA, EA_N, GB_Other, SC_Tai_Viet, LB_SA, EA_N, GB_Other, SC_Tai_Viet,
    LB_SA, EA_N, GB_Other, SC_Tai_Viet, LB_SA, EA_N, GB_Other, SC_Tai_Viet,
    LB_SA, EA_N, GB_Other, SC_Tai_Viet, LB_SA, EA_N, GB_Other, SC_Tai_Viet,
    LB_SA, EA_N, GB_Other, SC_Tai_Viet, LB_SA, EA_N, GB_Other, SC_Tai_Viet,
    LB_SA, EA_N, GB_Other, SC_Tai_Viet, LB_SA, EA_N, GB_Other, SC_Tai_Viet,
    LB_SA, EA_N, GB_Other, SC_Tai_Viet, LB_SA, EA_N, GB_Other, SC_Tai_Viet,
    LB_SA, EA_N, GB_Other, SC_Tai_Viet, LB_SA, EA_N, GB_Other, SC_Tai_Viet,
    LB_SA, EA_N, GB_Other, SC_Tai_Viet, LB_SA, EA_N, GB_Other, SC_Tai_Viet,
    LB_SA, EA_N, GB_Other, SC_Tai_Viet, LB_SA, EA_N, GB_Other, SC_Tai_Viet,
    LB_SA, EA_N, GB_Other, SC_Tai_Viet, LB_SA, EA_N, GB_Other, SC_Tai_Viet,
    LB_SA, EA_N, GB_Other, SC_Tai_Viet, LB_SA, EA_N, GB_Other, SC_Tai_Viet,
    LB_SA, EA_N, GB_Other, SC_Tai_Viet, LB_SA, EA_N, GB_Other, SC_Tai_Viet,
    LB_SA, EA_N, GB_Other, SC_Tai_Viet, LB_SA, EA_N, GB_Other, SC_Tai_Viet,
    LB_SA, EA_Z, GB_Extend, SC_Tai_Viet, LB_SA, EA_N, GB_Other, SC_Tai_Viet,
    LB_SA, EA_Z, GB_Extend, SC_Tai_Viet, LB_SA, EA_Z, GB_Extend, SC_Tai_Viet,
    LB_SA, EA_Z, GB_Extend, SC_Tai_Viet, LB_SA, EA_N, GB_Other, SC_Tai_Viet,
    LB_SA, EA_N, GB_Other, SC_Tai_Viet, LB_SA, EA_Z, GB_Extend, SC_Tai_Viet,
    LB_SA, EA_Z, GB_Extend, SC_Tai_Viet, LB_SA, EA_N, GB_Other, SC_Tai_Viet,
    LB_SA, EA_N, GB_Other, SC_Tai_Viet, LB_SA, EA_N, GB_Other, SC_Tai_Viet,
    LB_SA, EA_N, GB_Other, SC_Tai_Viet, LB_SA, EA_N, GB_Other, SC_Tai_Viet,
    LB_SA, EA_Z, GB_Extend, SC_Tai_Viet, LB_SA, EA_Z, GB_Extend, SC_Tai_Viet,
    LB_SA, EA_N, GB_Other, SC_Tai_Viet, LB_SA, EA_Z, GB_Extend, SC_Tai_Viet,
    LB_SA, EA_N, GB_Other, SC_Tai_Viet, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_SA, EA_N, GB_Other, SC_Tai_Viet,
    LB_SA, EA_N, GB_Other, SC_Tai_Viet, LB_SA, EA_N, GB_Other, SC_Tai_Viet,
    LB_SA, EA_N, GB_Other, SC_Tai_Viet, LB_SA, EA_N, GB_Other, SC_Tai_Viet,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_Z,
    GB_Virama, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM,
    EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_Z, GB_Virama, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_H2,
    EA_W, GB_LV, SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W,
    GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H2, EA_W, GB_LV,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_JV, EA_W, GB_V,
    SC_Unknown, LB_JV, EA_W, GB_V, SC_Unknown, LB_JV, EA_W, GB_V, SC_Unknown,
    LB_JV, EA_W, GB_V, SC_Unknown, LB_JV, EA_W, GB_V, SC_Unknown, LB_JV,
    EA_W, GB_V, SC_Unknown, LB_JV, EA_W, GB_V, SC_Unknown, LB_JV, EA_W, GB_V,
    SC_Unknown, LB_JV, EA_W, GB_V, SC_Unknown, LB_JV, EA_W, GB_V, SC_Unknown,
    LB_JV, EA_W, GB_V, SC_Unknown, LB_JV, EA_W, GB_V, SC_Unknown, LB_JV,
    EA_W, GB_V, SC_Unknown, LB_JV, EA_W, GB_V, SC_Unknown, LB_JV, EA_W, GB_V,
    SC_Unknown, LB_JV, EA_W, GB_V, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_JT, EA_W, GB_T,
    SC_Unknown, LB_JT, EA_W, GB_T, SC_Unknown, LB_JT, EA_W, GB_T, SC_Unknown,
    LB_JT, EA_W, GB_T, SC_Unknown, LB_JT, EA_W, GB_T, SC_Unknown, LB_JT,
    EA_W, GB_T, SC_Unknown, LB_JT, EA_W, GB_T, SC_Unknown, LB_JT, EA_W, GB_T,
    SC_Unknown, LB_JT, EA_W, GB_T, SC_Unknown, LB_JT, EA_W, GB_T, SC_Unknown,
    LB_JT, EA_W, GB_T, SC_Unknown, LB_JT, EA_W, GB_T, SC_Unknown, LB_JT,
    EA_W, GB_T, SC_Unknown, LB_JT, EA_W, GB_T, SC_Unknown, LB_JT, EA_W, GB_T,
    SC_Unknown, LB_JT, EA_W, GB_T, SC_Unknown, LB_JT, EA_W, GB_T, SC_Unknown,
    LB_JT, EA_W, GB_T, SC_Unknown, LB_JT, EA_W, GB_T, SC_Unknown, LB_JT,
    EA_W, GB_T, SC_Unknown, LB_JT, EA_W, GB_T, SC_Unknown, LB_JT, EA_W, GB_T,
    SC_Unknown, LB_JT, EA_W, GB_T, SC_Unknown, LB_JT, EA_W, GB_T, SC_Unknown,
    LB_JT, EA_W, GB_T, SC_Unknown, LB_JT, EA_W, GB_T, SC_Unknown, LB_JT,
    EA_W, GB_T, SC_Unknown, LB_JT, EA_W, GB_T, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Control, SC_Unknown, LB_AL, EA_N, GB_Control, SC_Unknown, LB_AL, EA_N,
    GB_Control, SC_Unknown, LB_AL, EA_N, GB_Control, SC_Unknown, LB_AL, EA_N,
    GB_Control, SC_Unknown, LB_AL, EA_N, GB_Control, SC_Unknown, LB_AL, EA_N,
    GB_Control, SC_Unknown, LB_AL, EA_N, GB_Control, SC_Unknown, LB_AL, EA_N,
    GB_Control, SC_Unknown, LB_AL, EA_N, GB_Control, SC_Unknown, LB_AL, EA_N,
    GB_Control, SC_Unknown, LB_AL, EA_N, GB_Control, SC_Unknown, LB_AL, EA_N,
    GB_Control, SC_Unknown, LB_AL, EA_N, GB_Control, SC_Unknown, LB_AL, EA_N,
    GB_Control, SC_Unknown, LB_AL, EA_N, GB_Control, SC_Unknown, LB_AL, EA_N,
    GB_Control, SC_Unknown, LB_AL, EA_N, GB_Control, SC_Unknown, LB_AL, EA_N,
    GB_Control, SC_Unknown, LB_AL, EA_N, GB_Control, SC_Unknown, LB_AL, EA_N,
    GB_Control, SC_Unknown, LB_AL, EA_N, GB_Control, SC_Unknown, LB_AL, EA_N,
    GB_Control, SC_Unknown, LB_AL, EA_N, GB_Control, SC_Unknown, LB_AL, EA_N,
    GB_Control, SC_Unknown, LB_AL, EA_N, GB_Control, SC_Unknown, LB_AL, EA_N,
    GB_Control, SC_Unknown, LB_AL, EA_N, GB_Control, SC_Unknown, LB_AL, EA_N,
    GB_Control, SC_Unknown, LB_AL, EA_N, GB_Control, SC_Unknown, LB_AL, EA_N,
    GB_Control, SC_Unknown, LB_AL, EA_N, GB_Control, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_HL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_HL, EA_N, GB_Other, SC_Unknown, LB_HL, EA_N,
    GB_Other, SC_Unknown, LB_HL, EA_N, GB_Other, SC_Unknown, LB_HL, EA_N,
    GB_Other, SC_Unknown, LB_HL, EA_N, GB_Other, SC_Unknown, LB_HL, EA_N,
    GB_Other, SC_Unknown, LB_HL, EA_N, GB_Other, SC_Unknown, LB_HL, EA_N,
    GB_Other, SC_Unknown, LB_HL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_HL, EA_N, GB_Other, SC_Unknown, LB_HL, EA_N,
    GB_Other, SC_Unknown, LB_HL, EA_N, GB_Other, SC_Unknown, LB_HL, EA_N,
    GB_Other, SC_Unknown, LB_HL, EA_N, GB_Other, SC_Unknown, LB_HL, EA_N,
    GB_Other, SC_Unknown, LB_HL, EA_N, GB_Other, SC_Unknown, LB_HL, EA_N,
    GB_Other, SC_Unknown, LB_HL, EA_N, GB_Other, SC_Unknown, LB_HL, EA_N,
    GB_Other, SC_Unknown, LB_HL, EA_N, GB_Other, SC_Unknown, LB_HL, EA_N,
    GB_Other, SC_Unknown, LB_HL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_HL, EA_N, GB_Other, SC_Unknown, LB_HL, EA_N,
    GB_Other, SC_Unknown, LB_HL, EA_N, GB_Other, SC_Unknown, LB_HL, EA_N,
    GB_Other, SC_Unknown, LB_HL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_HL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_HL, EA_N, GB_Other, SC_Unknown, LB_HL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_HL, EA_N,
    GB_Other, SC_Unknown, LB_HL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_HL, EA_N, GB_Other, SC_Unknown, LB_HL, EA_N,
    GB_Other, SC_Unknown, LB_HL, EA_N, GB_Other, SC_Unknown, LB_HL, EA_N,
    GB_Other, SC_Unknown, LB_HL, EA_N, GB_Other, SC_Unknown, LB_HL, EA_N,
    GB_Other, SC_Unknown, LB_HL, EA_N, GB_Other, SC_Unknown, LB_HL, EA_N,
    GB_Other, SC_Unknown, LB_HL, EA_N, GB_Other, SC_Unknown, LB_HL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_OP, EA_N, GB_Other, SC_Unknown, LB_CL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_PO, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_IS, EA_W, GB_Other, SC_Unknown, LB_CL, EA_W,
    GB_Other, SC_Unknown, LB_CL, EA_W, GB_Other, SC_Unknown, LB_IS, EA_W,
    GB_Other, SC_Unknown, LB_IS, EA_W, GB_Other, SC_Unknown, LB_EX, EA_W,
    GB_Other, SC_Unknown, LB_EX, EA_W, GB_Other, SC_Unknown, LB_OP, EA_W,
    GB_Other, SC_Unknown, LB_CL, EA_W, GB_Other, SC_Unknown, LB_IN, EA_W,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID, EA_W,
    GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID, EA_W,
    GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_OP, EA_W,
    GB_Other, SC_Unknown, LB_CL, EA_W, GB_Other, SC_Unknown, LB_OP, EA_W,
    GB_Other, SC_Unknown, LB_CL, EA_W, GB_Other, SC_Unknown, LB_OP, EA_W,
    GB_Other, SC_Unknown, LB_CL, EA_W, GB_Other, SC_Unknown, LB_OP, EA_W,
    GB_Other, SC_Unknown, LB_CL, EA_W, GB_Other, SC_Unknown, LB_OP, EA_W,
    GB_Other, SC_Unknown, LB_CL, EA_W, GB_Other, SC_Unknown, LB_OP, EA_W,
    GB_Other, SC_Unknown, LB_CL, EA_W, GB_Other, SC_Unknown, LB_ID, EA_W,
    GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_OP, EA_W,
    GB_Other, SC_Unknown, LB_CL, EA_W, GB_Other, SC_Unknown, LB_ID, EA_W,
    GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID, EA_W,
    GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID, EA_W,
    GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID, EA_W,
    GB_Other, SC_Unknown, LB_CL, EA_W, GB_Other, SC_Unknown, LB_ID, EA_W,
    GB_Other, SC_Unknown, LB_CL, EA_W, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_NS, EA_W, GB_Other, SC_Unknown, LB_NS, EA_W,
    GB_Other, SC_Unknown, LB_EX, EA_W, GB_Other, SC_Unknown, LB_EX, EA_W,
    GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_OP, EA_W,
    GB_Other, SC_Unknown, LB_CL, EA_W, GB_Other, SC_Unknown, LB_OP, EA_W,
    GB_Other, SC_Unknown, LB_CL, EA_W, GB_Other, SC_Unknown, LB_OP, EA_W,
    GB_Other, SC_Unknown, LB_CL, EA_W, GB_Other, SC_Unknown, LB_ID, EA_W,
    GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID, EA_W,
    GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID, EA_W,
    GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID, EA_W,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_ID, EA_W,
    GB_Other, SC_Unknown, LB_PR, EA_W, GB_Other, SC_Unknown, LB_PO, EA_W,
    GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_WJ, EA_Z, GB_Control, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_EX, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F,
    GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_PR, EA_F,
    GB_Other, SC_Unknown, LB_PO, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F,
    GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_OP, EA_F,
    GB_Other, SC_Unknown, LB_CL, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F,
    GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_CL, EA_F,
    GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_CL, EA_F,
    GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F,
    GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F,
    GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F,
    GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F,
    GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F,
    GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_NS, EA_F,
    GB_Other, SC_Unknown, LB_NS, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F,
    GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F,
    GB_Other, SC_Unknown, LB_EX, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F,
    GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F,
    GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F,
    GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F,
    GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F,
    GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F,
    GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F,
    GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F,
    GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F,
    GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F,
    GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F,
    GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F,
    GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F,
    GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F,
    GB_Other, SC_Unknown, LB_OP, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F,
    GB_Other, SC_Unknown, LB_CL, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F,
    GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F,
    GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F,
    GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F,
    GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F,
    GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F,
    GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F,
    GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F,
    GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F,
    GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F,
    GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F,
    GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F,
    GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F,
    GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F,
    GB_Other, SC_Unknown, LB_OP, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F,
    GB_Other, SC_Unknown, LB_CL, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F,
    GB_Other, SC_Unknown, LB_OP, EA_F, GB_Other, SC_Unknown, LB_CL, EA_F,
    GB_Other, SC_Unknown, LB_CL, EA_H, GB_Other, SC_Unknown, LB_OP, EA_H,
    GB_Other, SC_Unknown, LB_CL, EA_H, GB_Other, SC_Unknown, LB_CL, EA_H,
    GB_Other, SC_Unknown, LB_NS, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_CJ, EA_H, GB_Other, SC_Unknown, LB_CJ, EA_H,
    GB_Other, SC_Unknown, LB_CJ, EA_H, GB_Other, SC_Unknown, LB_CJ, EA_H,
    GB_Other, SC_Unknown, LB_CJ, EA_H, GB_Other, SC_Unknown, LB_CJ, EA_H,
    GB_Other, SC_Unknown, LB_CJ, EA_H, GB_Other, SC_Unknown, LB_CJ, EA_H,
    GB_Other, SC_Unknown, LB_CJ, EA_H, GB_Other, SC_Unknown, LB_CJ, EA_H,
    GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_NS, EA_H, GB_Extend, SC_Unknown, LB_NS, EA_H,
    GB_Extend, SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_PO, EA_F,
    GB_Other, SC_Unknown, LB_PR, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F,
    GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F,
    GB_Other, SC_Unknown, LB_PR, EA_F, GB_Other, SC_Unknown, LB_PR, EA_F,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Control, SC_Unknown, LB_AL, EA_N, GB_Control, SC_Unknown, LB_AL, EA_N,
    GB_Control, SC_Unknown, LB_AL, EA_N, GB_Control, SC_Unknown, LB_AL, EA_N,
    GB_Control, SC_Unknown, LB_AL, EA_N, GB_Control, SC_Unknown, LB_AL, EA_N,
    GB_Control, SC_Unknown, LB_AL, EA_N, GB_Control, SC_Unknown, LB_AL, EA_N,
    GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z,
    GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CB, EA_N,
    GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL,
    EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL,
    EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM,
    EA_Z, GB_Virama, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Virama,
    SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_Z, GB_Virama, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_Z, GB_Control, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Virama, SC_Unknown, LB_CM, EA_Z, GB_Virama, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N,
    GB_Virama, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL,
    EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_BA, EA_N,
    GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N,
    GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N,
    GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N,
    GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N,
    GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL,
    EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter,
    SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown, LB_AL, EA_N,
    GB_OtherLetter, SC_Unknown, LB_AL, EA_N, GB_OtherLetter, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_N, GB_Virama, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown,
    LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_OP, EA_N, GB_Other, SC_Unknown, LB_OP, EA_N, GB_Other, SC_Unknown,
    LB_OP, EA_N, GB_Other, SC_Unknown, LB_CL, EA_N, GB_Other, SC_Unknown,
    LB_CL, EA_N, GB_Other, SC_Unknown, LB_CL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_CL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_OP, EA_N, GB_Other, SC_Unknown, LB_CL, EA_N, GB_Other, SC_Unknown,
    LB_OP, EA_N, GB_Other, SC_Unknown, LB_CL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_OP, EA_N, GB_Other, SC_Unknown,
    LB_CL, EA_N, GB_Other, SC_Unknown, LB_CL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_CM, EA_N, GB_Extend, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_N, GB_Extend, SC_Unknown, LB_CM, EA_N, GB_Extend, SC_Unknown,
    LB_CM, EA_N, GB_Extend, SC_Unknown, LB_CM, EA_N, GB_Extend, SC_Unknown,
    LB_CM, EA_N, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown,
    LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown,
    LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown,
    LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown,
    LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown,
    LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown,
    LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown,
    LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown,
    LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown,
    LB_ID, EA_W, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown
};

